/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2010 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE 
SERVICES. IN NO EVENT SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL 
DAMAGES ARISING OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            stnp002
* Version:          1.0
* Date:             Feb-24-2011
* Classification:   General Business Information
* Brief:            eTimer-ADC interrupt demo
********************************************************************************
* Detailed Description: 
* 
* This example shows ADC triggering from the eTimer1 module.
* The OFLAG signal from the eTimer1 channel 5 is fed into both ADC converters, 
* So ADC is set up for injected conversion with end of scan interrupt. 
* 
* ------------------------------------------------------------------------------
* Test HW:  MPC5604PKIT
* Maskset:  0M36W
* Target :  internal_RAM
* Terminal: no
* Fsys:     64 MHz with 40 MHz XOSC reference
* Debugger:  IDCPPCNEXUS
*
* EVB connection:
*
*	PJ7.14 - B[13] .. ADC1 AN[0] input
*   PJ9.6  - A[5]  .. eTimer1 channel5 OFLAG signal
*   PJ8.1  - D[0]  .. GPIO output, used to show ADC interrupt routine execution
*
* connect Trimmer J18.2 to PJ7.14 to change eTimer1 channle5 output period
*
********************************************************************************
Revision History:
1.0     Feb-24-2010     stnp002  Initial Version
*******************************************************************************//**/


#include "MPC5604P_M26V.h"
/*******************************************************************************
* Global variables
*******************************************************************************/
uint16_t result;

/*******************************************************************************
* Constants and macros
*******************************************************************************/

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void InitModesAndClks(void);
static void InitPeriClkGen(void);
static void DisableWatchdog(void);
static void InitHW(void);
static void InitGPIO(void);
/*******************************************************************************
* Local variables
*******************************************************************************/

/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : InitHW
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : init hardware for the purposes of this example code
Issues        : NONE
*******************************************************************************/
static void InitHW(void)
{
    DisableWatchdog();
    InitModesAndClks();
    InitPeriClkGen();     
}

/*******************************************************************************
Function Name : InitModesAndClks
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Enter RUN0 with Fsys 64 MHz PLL with 40 MHz crystal reference.
                monitor Fsys on CLKOUT pin = PB6 = PJ7.7
Issues        : NONE
*******************************************************************************/
static void InitModesAndClks(void) 
{
  	ME.MER.R = 0x0000001D;          /* Enable DRUN, RUN0, SAFE, RESET modes */
                                  /* Initialize PLL before turning it on: */
	/* Use 2 of the next 4 lines depending on crystal frequency: */
	/*CGM.CMU_0_CSR.R = 0x000000004; */ /* Monitor FXOSC > FIRC/4 (4MHz); no PLL monitor */
	/*CGM.FMPLL[0].CR.R = 0x02400100;*/ /* 8 MHz xtal: Set PLL0 to 64 MHz */   
  	CGM.CMU_0_CSR.R = 0x000000000;  /* Monitor FXOSC > FIRC/1 (16MHz); no PLL monitor*/
  	CGM.FMPLL[0].CR.R = 0x12400100; /* 40 MHz xtal: Set PLL0 to 64 MHz */   
  	ME.RUN[0].R = 0x001F0074;       /* RUN0 cfg: 16MHzIRCON,OSC0ON,PLL0ON,syclk=PLL0 */
  	ME.RUNPC[0].R = 0x000000FE; 	  /* Peri. Cfg. 0 settings: run in all modes */
  	ME.PCTL[4].R  = 0x00; 	      /* MPC56xxB/P/S DSPI0: select ME.RUNPC[0] */
  	ME.PCTL[32].R = 0x00; 	      /* MPC56xxB/P/S ADC 0: select ME.RUNPC[0] */
  	ME.PCTL[33].R = 0x00; 	      /* MPC56xxB/P/S ADC 1: select ME.RUNPC[0] */
  	ME.PCTL[35].R = 0x00; 	      /* MPC56xxB/P/S CTU 0: select ME.RUNPC[0] */
  	ME.PCTL[41].R = 0x00; 	      /* MPC56xxB/P/S FlexPWM: select ME.RUNPC[0] */	
                                  /* Mode Transition to enter RUN0 mode: */
  	ME.MCTL.R = 0x40005AF0;         /* Enter RUN0 Mode & Key */
  	ME.MCTL.R = 0x4000A50F;         /* Enter RUN0 Mode & Inverted Key */  
  	while (ME.GS.B.S_MTRANS) {}     /* Wait for mode transition to complete */    
                                  /* Note: could wait here using timer and/or I_TC IRQ */
  	while(ME.GS.B.S_CURRENTMODE != 4) {} /* Verify RUN0 is the current mode */
    
    
    SIU.PCR[22].R = 0x0600;		/* enable CLKOUT on PB6 */
    
    /* set CLKOUT divider of 4 */
    CGM.OCDSSC.B.SELDIV = 0b10;  /* output selected Output Clock divided by 4 */
    CGM.OCDSSC.B.SELCTL = 0b0010;  /* fmpll_0 */
    CGM.OCEN.B.EN = 1; 			 /* enable CLKOUT signal */
}

/*******************************************************************************
Function Name : InitPeriClkGen
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Enable auxiliary clock 0
Issues        : NONE
*******************************************************************************/
static void InitPeriClkGen(void) 
{
    CGM.AC0SC.R = 0x04000000;  /* MPC56xxP ADC: Select IPBClk for aux clk 0  */
    CGM.AC0DC.R = 0x80000000;  /* MPC56xxP ADC: Enable aux clk 0 div by 1 */
} 

/*******************************************************************************
Function Name : InitGPIO
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : IO configuration
Issues        : NONE
*******************************************************************************/
static void InitGPIO(void)
{
             
    SIU.PCR[5].R = 0x0A00;	// A[5] pin as eTimer1 channel5 output
    SIU.PCR[23].R = 0x2400; // B[7] pin as ADC0 AN[0] input
    SIU.PCR[29].R = 0x2400; // B[13] pin as ADC1 AN[0] input
    SIU.PCR[48].R = 0x0200; // D[0] pin as GPIO output
}

/*******************************************************************************
Function Name : DisableWatchdog
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : disable SWT functionality
Issues        : NONE
*******************************************************************************/
static void DisableWatchdog(void) 
{
    SWT.SR.R = 0x0000c520;     /* Write keys to clear soft lock bit */
    SWT.SR.R = 0x0000d928; 
    SWT.CR.R = 0xFF00010A;     /* Clear watchdog enable (WEN) */
}

/*******************************************************************************
Function Name : InitETimer
Engineer      : stnp002
Date          : Feb-24-2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : eTimer0 channel 1 output compare every 0.5 ms
Issues        : NONE
*******************************************************************************/
static void InitETimer(void)
{
    /* 0x7D00 is for 1 ms period when running on 64 MHz PLL */
    ETIMER_1.CHANNEL[5].COMP1.R = 0x7D00;			// Count untill this value, then Reinitialize to 0x0000 (0x0190 = 400 --> 16MhZ/400 = 40kHz --> toggle every 25s)(0x3e80 = 16000 --> 16MhZ/16000 = 1kHz --> toggle every ms)
    ETIMER_1.CHANNEL[5].CMPLD1.R = 0x7D00;
    ETIMER_1.CHANNEL[5].COMP2.R = 0xfff0;			// Because COMP2 is greater, this will not affect the reinitialize
    ETIMER_1.CHANNEL[5].CTRL.R = 0x3840;			// Counts only rising edge of the Fsys (64MHz in RUN0), divide by prescaler, count up untill Comp then reinit, count up, count repeatedly
    ETIMER_1.CHANNEL[5].CTRL2.R = 0x8003;			// OFLAG is driven to external pin, toggle oflag on successful compare to comp1 & 2 
    ETIMER_1.CHANNEL[5].CCCTRL.R = 0x0A00;			// Load COMP1 with CMPLD1 on compare, compare with COMP1/COMP2 if counting up/dowm
   // ETIMER_1.CHANNEL[5].CTRL3.R = 0x0f00;
   // ETIMER_1.CHANNEL[5].INTDMA.R = 0x0002;			// Timer Compare flag1 (TCF1) interrupt enable, interrupt when count = compare1 reg
   // ETIMER_1.CHANNEL[5].STS.R = 0x0002;   		 	// clear interrupt flag by writing one

    ETIMER_1.ENBL.R = 0x0020;					// Enable Timer1 channels 5
}

/*******************************************************************************
Function Name : InitADC
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : ADC initialization
Issues        : NONE
*******************************************************************************/
static void InitADC(void)
{

    //------------------------
    // ADC 1 initialization
    //------------------------
    ADC_1.MCR.R	= 0x80600000;		// enable result overwriting, enable External trigger (rising edge) for channel injection
    ADC_1.CTR[0].R = 0x00008606;	// 
    ADC_1.JCMR[0].B.CH0 = 0x1;		// sampling is enabled for channel 0
    ADC_1.IMR.B.MSKJEOC = 0x1;	// End of Injected Channel Conversion is enabled
    ADC_1.CIMR[0].B.CIM0 = 0x1;	// interrupt for channel 0 is enabled

}

/******************************************************************************
Function Name : ADC1_EOC_isr
Engineer      : stnp002
Date          : Feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : ADC end of scan interrupt routine
Issues        : NONE
*******************************************************************************/
void ADC1_EOC_isr(void)
{
	
	result = (uint16_t)ADC_1.CDR[0].B.CDATA;     /* Read ADC1 chan0 conversion result data */
	ETIMER_1.CHANNEL[5].CMPLD1.R = 0x7D00 + (result<<4);
	
	ADC_1.ISR.B.JEOC = 0x1;		// clear JEOC flag
	
	SIU.GPDO[48].R ^= 1;		// toggle D[0] pin
	
}

/*******************************************************************************
Function Name : main
Engineer      : stnp002
Date          : feb-24-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : main function
Issues        : NONE
*******************************************************************************/
int32_t main(void) 
{
    int32_t i = 0;
    register int32_t n = 0;

    InitHW();  
 
    InitGPIO();
    InitETimer();
    InitADC();
    
    INTC_InstallINTCInterruptHandler(ADC1_EOC_isr, 82,2); /* install an interrupt handler (ADC_EOC) 
  														  for a given interrupt vector (82) and 
  														  set the interrupt priority (2) */
  
    INTC.CPR.B.PRI = 0;	/* Lower INTC's current priority, by default highest, e.g 15, is used */
    
    /* Loop forever */
    for (;;) 
    {
        i++;
    }
}



